#                                                         -*- Perl -*-
# Copyright (C) 2001  Kazuhiko  <kazuhiko@ring.gr.jp>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

@replace_list = ('', '', ',', 'f', ',,', '...', '', '',
		 '^', '', 'S', '<', 'OE', '', 'Z', '',
		 '', '`', '\'', '``', '\'\'', '', '-', '-',
		 '~', 'TM', 's', '>', 'oe', '', 'z', 'Y',
		 ' ', '!', '', '', '', '\\', '|', '',
		 '\"', 'C', 'a', '<<', '', '-', 'R', '~',
		 '', '+-', '2', '3', '\'', '', '', '',
		 '', '1', 'o', '>>', '1/4', '1/2', '3/4', '?',
		 'A', 'A', 'A', 'A', 'A', 'A', 'AE', 'C',
		 'E', 'E', 'E', 'E', 'I', 'I', 'I', 'I',
		 'D', 'N', 'O', 'O', 'O', 'O', 'O', '',
		 'O', 'U', 'U', 'U', 'U', 'Y', 'TH', 'ss',
		 'a', 'a', 'a', 'a', 'a', 'a', 'ae', 'c',
		 'e', 'e', 'e', 'e', 'i', 'i', 'i', 'i',
		 'o', 'n', 'o', 'o', 'o', 'o', 'o', '',
		 'o', 'u', 'u', 'u', 'u', 'y', 'th', 'y');

sub fpw_add_entry_latin1 {
    my ($obj, $item) = @_;
    my $header = '';

    foreach $char (split //, $item) {
	if (ord $char < 0x80) {
	    $header .= $char;
	} else {
	    $header .= $replace_list[(ord $char) - 0x80];
	}
    }
    $obj->add_entry($header, $heading_position, $text_position)
      || fpw_error($obj);
    return $header;
}

sub fpw_add_text_latin1 {
    my ($obj, $item) = @_;
    foreach $char (split //,$item) {
	if (ord $char < 0x80) {
	    #
	    # ASCII character ν񤭹
	    #
	    $obj->add_text($char) ||
	      die "$PROGRAM_NAME: " . $obj->error_message() . "\n";
	} else {
	    #
	    # Latin 1 character ν񤭹 (Ⱦѳ)
	    #
	    $gaiji = sprintf "%02x", ord $char;
	    $obj->add_half_user_character($gaiji) ||
	      die "$PROGRAM_NAME: " . $obj->error_message() . "\n";
	}
    }
}

sub fpw_add_entry {
    my ($obj, $item) = @_;

    $obj->add_entry($item, $heading_position, 'head',
		    $text_position, 'text') || fpw_error($obj);
    return $item;
}

sub fpw_add_text {
    my ($obj, $item) = @_;
    $obj->add_text($item) || fpw_error($obj);
    return $item;
}

sub fpw_add_newline {
    my $obj = $_[0];
    $obj->add_newline() || fpw_error($obj);
}

sub fpw_add_keyword_latin1 {
    my ($obj, $item) = @_;
    $obj->add_keyword_start() || fpw_error($obj);
    fpw_add_text_latin1($obj, $item);
    if (!$obj->add_keyword_end()
	|| !$obj->add_newline()
	|| !$obj->add_indent_level(2)) {
	fpw_error($obj);
    }
}

sub fpw_add_keyword {
    my ($obj, $item) = @_;
    $obj->add_keyword_start() || fpw_error($obj);
    fpw_add_text($obj, $item);
    if (!$obj->add_keyword_end()
	|| !$obj->add_newline()
	|| !$obj->add_indent_level(2)) {
	fpw_error($obj);
    }
}

sub fpw_add_entry_tag {
    my ($obj, $item) = @_;
    $obj->add_entry_tag(fpw_tag($item)) || fpw_error($obj);
}

sub fpw_add_reference_start {
    my $obj = $_[0];
    $obj->add_reference_start() || fpw_error($obj);
}

sub fpw_add_reference_end {
    my ($obj, $item) = @_;
    $obj->add_reference_end(fpw_tag($item)) || fpw_error($obj);
}

sub fpw_add_new_entry {
    my $obj = $_[0];
    $obj->new_entry() || fpw_error($obj);
}

sub fpw_add_menu {
    my ($obj, $item) = @_;
    if (!$obj->add_reference_start()
	|| !$obj->add_text($item)
	|| !$obj->add_reference_end(fpw_tag($item))
	|| !$obj->add_newline()) {
	fpw_error($obj);
    }
}

sub fpw_add_menu_entry {
    my ($obj, $item) = @_;
    if (!$obj->new_context()
	|| !$obj->add_tag(fpw_tag($item))) {
	fpw_error($obj);
    }
}

sub fpw_add_color_graphic_start {
    my ($obj, $item) = @_;
    $obj->add_color_graphic_start($item) || fpw_error($obj);
}

sub fpw_add_color_graphic_end {
    my $obj = $_[0];
    $obj->add_color_graphic_end() || fpw_error($obj);
}

sub fpw_add_color_jpeg_start {
    my ($obj, $item) = @_;
    $obj->add_color_jpeg_start($item) || fpw_error($obj);
}

sub fpw_add_color_jpeg_end {
    my $obj = $_[0];
    $obj->add_color_jpeg_end() || fpw_error($obj);
}

sub fpw_error {
  my($obj) = $_[0];
  die "$PROGRAM_NAME: " . $obj->error_message() . "\n";
}

sub fpw_tag {
    my $item = $_[0];
    return "T" . unpack "H*", $item;
}

sub fpw_z2h {
  my($obj) = jcode($_[0]);
  $obj = $obj->tr("ɡǡʡˡܡݡ","!\"#\$%&'()*+,-./");
  $obj = $obj->tr("䡩ΡϡơСá",":;<=>?[\]^_`{|}");
  $obj = $obj->tr("-","0-9");
  $obj = $obj->tr("-","A-Z");
  $obj = $obj->tr("-","a-z");
  return $obj;
}
